@@ -12,7 +12,7 @@
 #         stat.S_ISFIFO(mode)   == FIFO (named pipe)
 #         stat.S_ISSOCK(mode)   == socket
 
-import os, six, asset, stat, collections
+import sys, os, six, asset, stat, collections
 
 #------------------------------------------------------------------------------
 class UnknownOverlayMode(Exception): pass
@@ -64,8 +64,13 @@ class OverlayEntry(object):
     if self.mode is None:
       raise OSError(2, 'No such file or directory', self.path)
     size = len(self.content or '')
+    mode = self.mode
+    if mode == stat.S_IFDIR:
+      mode |= stat.S_IRWXU
+    else:
+      mode |= stat.S_IRUSR | stat.S_IWUSR
     return OverlayStat(
-      st_mode=self.mode, st_size=size, st_overlay=1,
+      st_mode=mode, st_size=size, st_overlay=1,
       # todo: if possible, inherit these from the original entry...
       st_ino=0, st_dev=0, st_nlink=0, st_uid=0, st_gid=0,
       st_atime=0, st_mtime=0, st_ctime=0)
@@ -107,6 +112,8 @@ class FileSystemOverlay(object):
     'os:access'         : 'fso_access',
     'os.path:exists'    : 'fso_exists',
     'os.path:lexists'   : 'fso_lexists',
+    'os.path:islink'    : 'fso_islink',
+    'shutil:rmtree'     : 'fso_rmtree',
     }
 
   #----------------------------------------------------------------------------
@@ -314,6 +321,20 @@ class FileSystemOverlay(object):
     except os.error:
       return False
 
+  #----------------------------------------------------------------------------
+  def fso_access(self, path, mode):
+    try:
+      st = self.fso_stat(path)
+    except OSError:
+      return False
+    if mode & os.X_OK and not ( st.st_mode & ( stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH ) ):
+      return False
+    if mode & os.W_OK and not ( st.st_mode & ( stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH ) ):
+      return False
+    if mode & os.R_OK and not ( st.st_mode & ( stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH ) ):
+      return False
+    return True
+
   #----------------------------------------------------------------------------
   def fso_listdir(self, path):
     'overlays os.listdir()'
@@ -407,7 +428,55 @@ class FileSystemOverlay(object):
   #----------------------------------------------------------------------------
   def fso_remove(self, path):
     'overlays os.remove()'
-    return self.unlink(path)
+    return self.fso_unlink(path)
+
+  #----------------------------------------------------------------------------
+  def fso_islink(self, path):
+    'overlays os.path.islink()'
+    try:
+      return stat.S_ISLNK(self.fso_lstat(path).st_mode)
+    except OSError:
+      return False
+
+  #----------------------------------------------------------------------------
+  def fso_rmtree(self, path, ignore_errors=False, onerror=None):
+    'overlays shutil.rmtree()'
+    if ignore_errors:
+      def onerror(*args):
+        pass
+    elif onerror is None:
+      def onerror(*args):
+        raise
+    try:
+      if self.fso_islink(path):
+        # symlinks to directories are forbidden, see shutil bug #1669
+        raise OSError('Cannot call rmtree on a symbolic link')
+    except OSError:
+      onerror(os.path.islink, path, sys.exc_info())
+      # can't continue even if onerror hook returns
+      return
+    names = []
+    try:
+      names = self.fso_listdir(path)
+    except os.error, err:
+      onerror(os.listdir, path, sys.exc_info())
+    for name in names:
+      fullname = os.path.join(path, name)
+      try:
+        mode = self.fso_lstat(fullname).st_mode
+      except os.error:
+        mode = 0
+      if stat.S_ISDIR(mode):
+        self.fso_rmtree(fullname, ignore_errors, onerror)
+      else:
+        try:
+          self.fso_remove(fullname)
+        except OSError as err:
+          onerror(os.remove, fullname, sys.exc_info())
+    try:
+      self.fso_rmdir(path)
+    except os.error:
+      onerror(os.rmdir, path, sys.exc_info())
 
   #----------------------------------------------------------------------------
   def fso_open(self, path, mode=None, buffering=None):
@@ -495,17 +564,6 @@ class FileSystemOverlay(object):
     with self.originals['__builtin__:open'](path, 'rb') as fp:
       return OverlayFileStream(self, path, prepend=fp.read())
 
-  #############################################################################
-
-  #----------------------------------------------------------------------------
-  def fso_access(self, path, mode):
-    raise NotImplementedError()
-    # if path in self.entries:
-    #   # TODO: implement better file access control...
-    #   # TODO: make this dependent on entry.mode!...
-    #   return True
-    # return self.originals['os:access'](path, mode)
-
 #------------------------------------------------------------------------------
 # end of $Id$
 #------------------------------------------------------------------------------
