@@ -6,7 +6,11 @@
 # copy: (C) CopyLoose 2009 UberDev <hardcore@uberdev.org>, No Rights Reserved.
 #------------------------------------------------------------------------------
 
-import sys, os, unittest, tempfile, stat
+# IMPORTANT NOTE: in these routines, i am purposefully not doing a
+# shutil.rmtree() when doing a `os.rmdir()` so that the presence of
+# any unexpected files will cause an exception.
+
+import sys, os, unittest, tempfile, stat, shutil
 
 from .filesystemoverlay import FileSystemOverlay
 
@@ -84,8 +88,6 @@ class TestFileSystemOverlay(unittest.TestCase):
     for tfile in tfiles:
       self.assertFalse(os.path.exists(os.path.join(tdir, tfile)))
     self.assertEqual(os.listdir(tdir), [])
-    # note: purposefully not doing a shutil.rmtree() so that any unexpected
-    # files will cause an exception.
     os.rmdir(tdir)
 
   #----------------------------------------------------------------------------
@@ -152,6 +154,42 @@ class TestFileSystemOverlay(unittest.TestCase):
     os.rmdir(os.path.join(tdir, 'a'))
     os.rmdir(tdir)
 
+  #----------------------------------------------------------------------------
+  def test_rmtree(self):
+    tdir = tempfile.mkdtemp(prefix='fso-test_filesystemoverlay-unittest.rmtree.')
+    self.assertEqual(os.listdir(tdir), [])
+    os.makedirs(os.path.join(tdir, 'a/b/c/d'))
+    os.symlink('foo', os.path.join(tdir, 'a/b/c/d/bar'))
+    with open(os.path.join(tdir, 'a/b/c/file'), 'wb') as fp:
+      fp.write('data')
+    no_fso_walk = [
+      (tdir, ['a'], []),
+      (os.path.join(tdir, 'a'), ['b'], []),
+      (os.path.join(tdir, 'a/b'), ['c'], []),
+      (os.path.join(tdir, 'a/b/c'), ['d'], ['file']),
+      (os.path.join(tdir, 'a/b/c/d'), [], ['bar']),
+      ]
+    self.assertEqual(list(os.walk(tdir)), no_fso_walk)
+    with FileSystemOverlay() as fso:
+      self.assertEqual(list(os.walk(tdir)), no_fso_walk)
+      shutil.rmtree(os.path.join(tdir, 'a/b/c'))
+      self.assertEqual(list(os.walk(tdir)), [
+        (tdir, ['a'], []),
+        (os.path.join(tdir, 'a'), ['b'], []),
+        (os.path.join(tdir, 'a/b'), [], []),
+        ])
+      self.assertEqual(fso.changes, [
+        'del:' + os.path.join(tdir, 'a/b/c'),
+        'del:' + os.path.join(tdir, 'a/b/c/d'),
+        'del:' + os.path.join(tdir, 'a/b/c/d/bar'),
+        'del:' + os.path.join(tdir, 'a/b/c/file'),
+        ])
+    self.assertEqual(list(os.walk(tdir)), no_fso_walk)
+    shutil.rmtree(os.path.join(tdir, 'a/b/c'))
+    os.rmdir(os.path.join(tdir, 'a/b'))
+    os.rmdir(os.path.join(tdir, 'a'))
+    os.rmdir(tdir)
+
   #----------------------------------------------------------------------------
   def test_create(self):
     fname = tempfile.mktemp(prefix='fso-test_filesystemoverlay-unittest.create.')
@@ -165,6 +203,49 @@ class TestFileSystemOverlay(unittest.TestCase):
       self.assertEqual('temporary filename: ' + fname, open(fname, 'rb').read())
     self.assertFalse(os.path.exists(fname), 'write-through occurred')
 
+  #----------------------------------------------------------------------------
+  def test_access(self):
+    tdir = tempfile.mkdtemp(prefix='fso-test_filesystemoverlay-unittest.access.')
+    os.makedirs(os.path.join(tdir, 'a/b/c/d'))
+    os.symlink('foo',os.path.join(tdir, 'a/b/c/d/bar'))
+    with open(os.path.join(tdir, 'a/b/c/file'), 'wb') as fp:
+      fp.write('data')
+    self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.F_OK | os.R_OK | os.W_OK))
+    self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/file'), os.X_OK))
+    self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.W_OK))
+    self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.R_OK))
+    self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.F_OK))
+    self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.X_OK))
+    self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.W_OK))
+    self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.R_OK))
+    with FileSystemOverlay() as fso:
+      self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.F_OK | os.R_OK | os.W_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/file'), os.X_OK))
+      self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.W_OK))
+      self.assertTrue(os.access(os.path.join(tdir, 'a/b/c/file'), os.R_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.F_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.X_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.W_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/b/c/notfile'), os.R_OK))
+      os.mkdir(os.path.join(tdir, 'a/fso'))
+      with open(os.path.join(tdir, 'a/fso/file'), 'wb') as fp:
+        fp.write('fso.data')
+      self.assertTrue(os.access(os.path.join(tdir, 'a/fso/file'), os.F_OK | os.R_OK | os.W_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/fso/file'), os.X_OK))
+      self.assertTrue(os.access(os.path.join(tdir, 'a/fso/file'), os.W_OK))
+      self.assertTrue(os.access(os.path.join(tdir, 'a/fso/file'), os.R_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/fso/notfile'), os.F_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/fso/notfile'), os.X_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/fso/notfile'), os.W_OK))
+      self.assertFalse(os.access(os.path.join(tdir, 'a/fso/notfile'), os.R_OK))
+    os.unlink(os.path.join(tdir, 'a/b/c/d/bar'))
+    os.unlink(os.path.join(tdir, 'a/b/c/file'))
+    os.rmdir(os.path.join(tdir, 'a/b/c/d'))
+    os.rmdir(os.path.join(tdir, 'a/b/c'))
+    os.rmdir(os.path.join(tdir, 'a/b'))
+    os.rmdir(os.path.join(tdir, 'a'))
+    os.rmdir(tdir)
+
   #----------------------------------------------------------------------------
   def test_append(self):
     fname = tempfile.mktemp(prefix='fso-test_filesystemoverlay-unittest.append.')
